<?php
declare(strict_types=1);

namespace Catequil\Invoice\Service;

use Catequil\Invoice\Model\Invoice;
use Catequil\Invoice\Model\InvoiceInterface;
use Catequil\Webshop\Service\WebshopServiceInterface;
use Doctrine\ORM\EntityManagerInterface;
use Laminas\Hydrator\HydratorInterface;
use function intval;

/**
 * Class InvoiceService
 *
 * @package Catequil\Invoice\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-invoice
 */
final class InvoiceService implements InvoiceServiceInterface
{
    private EntityManagerInterface $em;
    private HydratorInterface $hydrator;
    private WebshopServiceInterface $webshops;

    /**
     * @param EntityManagerInterface $em
     * @param HydratorInterface $hydrator
     * @param WebshopServiceInterface $webshops
     */
    public function __construct(
        EntityManagerInterface $em,
        HydratorInterface $hydrator,
        WebshopServiceInterface $webshops
    ) {
        $this->em = $em;
        $this->hydrator = $hydrator;
        $this->webshops = $webshops;
    }

    public function findByBarcode(string $barcode): ?InvoiceInterface
    {
        if (!preg_match('/^([A-Z]+)(\d+)$/', $barcode, $matches)) {
            return null;
        }

        return $this->findByInvoiceNo($matches[1], intval($matches[2]));
    }

    public function findByInvoiceNo(string $documentType, int $invoiceNo): ?InvoiceInterface
    {
        return $this->em->getRepository(Invoice::class)
            ->createQueryBuilder('i')
            ->where('i.documentType = ?1')
            ->andWhere('i.invoiceNo = ?2')
            ->orderBy('i.createdAt', 'DESC')
            ->setMaxResults(1)
            ->getQuery()
            ->setParameter(1, $documentType)
            ->setParameter(2, $invoiceNo)
            ->getResult()[0] ?? null;
    }

    /**
     * @param array $data
     * @return InvoiceInterface
     */
    public function uploadFromGath(array $data): InvoiceInterface
    {
        $data['WebShopSiteName'] = $this->webshops->findByDomain($data['WebShopSiteName']);
        /** @var InvoiceInterface $invoice */
        $invoice = $this->hydrator->hydrate($data, new Invoice());

        $this->em->persist($invoice);
        $this->em->flush();

        return $invoice;
    }
}
